/**
  *  \file planetscentral/talk/bbparse.h
  *  \brief Talk: BBCode parser
  */
#ifndef C2SERVER_PLANETSCENTRAL_TALK_BBPARSE_H
#define C2SERVER_PLANETSCENTRAL_TALK_BBPARSE_H

#include "planetscentral/talk/text.h"
#include "cpluslib/ptrvector.h"
#include "planetscentral/talk/bblex.h"
#include "planetscentral/talk/smiley.h"

struct BBColor {
    const char* name;
    const char* rgb;

    static const BBColor* findColorByName(const char* name);
    static const BBColor* findColorByValue(const char* rgb);
};

class BBParser {
 public:
    BBParser(BBLexer& lex, InlineItemSet options);

    TextNode* parse();
    static bool isKnownTag(const string_t& tag);

 private:
    typedef ptr_vector<TextNode>::size_type depth_t;
    BBLexer& lex;
    InlineItemSet options;
    BBLexer::Token current;
    ptr_vector<TextNode> stack;

    void handleStart();
    void handleEnd();
    void handleSmiley();
    void handleParagraph();
    void handleAtLink();
    void handleText();

    void handleNoparse();
    void handleCode();
    void handleListItem();

    void next();
    void open(uint8_t major, uint8_t minor);
    void openAt(depth_t n, uint8_t major, uint8_t minor);
    void appendText(string_t what);
    void close();
    void close(depth_t n);
    void closeLinks();
    void closeInline();
    bool closeUntil(uint8_t major, uint8_t minor);
    bool inLink() const;
};

#endif
