/**
  *  \file planetscentral/talk/text.h
  *  \brief Talk: Representation of Text
  */
#ifndef C2SERVER_PLANETSCENTRAL_TALK_TEXT_H
#define C2SERVER_PLANETSCENTRAL_TALK_TEXT_H

#include "cpluslib/ptrvector.h"
#include "cpluslib/types.h"
#include "cpluslib/string.h"

/** Text node. This is our minimal "DOM" for manipulating forum text. */
struct TextNode {
    uint8_t major;              // major type, TextMajorKind
    uint8_t minor;              // minor type
    ptr_vector<TextNode> children;
    string_t text;

    bool isSimpleList() const;
    void stripQuotes();
    string_t rawTextContent();
};

enum TextMajorKind {
    maPlain,                    // Plain text, no children
    maInline,                   // Inline-formatting
    maInlineAttr,               // Inline-formatting with attribute
    maLink,                     // Link
    maParagraph,                // Paragraph. Contains text, inline format, links.
    maGroup,                    // Group. Contains paragraphs or groups.
    maSpecial                   // Specialties
};

enum TextInlineFormat {
    miInBold,                   // Bold
    miInItalic,                 // Italic
    miInStrikeThrough,          // Strikethrough
    miInUnderline,              // Underline
    miInMonospace               // Monospace
};

enum TextInlineAttrFormat {
    miIAColor,                  // Color (color in text, always in "#rrggbb" format)
    miIASize,                   // Size (size in text, always in "+nn" or "-nn" format)
    miIAFont                    // Font (font name in text)
};

enum TextLinkFormat {
    miLinkUrl,                  // URL (link target in text)
    miLinkEmail,                // Email (link target in text)
    miLinkThread,               // Thread link (thread Id in text)
    miLinkPost,                 // Post link (post Id in text)
    miLinkGame,                 // Game link (game Id in text)
    miLinkUser,                 // User link (user Id in text)
    miLinkForum                 // Forum link (forum Id in text)
};

enum TextParagraphFormat {
    miParNormal,                // normal paragraph
    miParCode,                  // [code], language in text
    miParCentered,              // [center]
    miParBreak                  // Cut mark for blog entries
};

enum TextGroupFormat {
    miGroupRoot,                // outermost group. Content is paragraphs or groups.
    miGroupQuote,               // reference in text. Content is paragraphs or groups.
    miGroupList,                // type in text. Content is miGroupListItem.
    miGroupListItem             // list items. Content is paragraphs.
};

enum TextSpecialFormat {
    miSpecialBreak,             // Line break
    miSpecialImage,             // Image (link in text, alt-text in content)
    miSpecialSmiley             // Smiley (name in text)
};

#endif
